/**
 * Timer class: calls the tick method every n milliseconds.
 *
 * <PRE>
 * Timer timer = new Timer(this,1000);  // 1 second interval
 * timer.start();
 * </PRE>
 *
 * @author Ignazio Locatelli
 */

 package it.MediaIllusion.Anim;

 public class Timer implements Runnable {

   // main thread
   protected Thread myThread = null;

   // milliseconds to wait
   protected long interval;

   // interface used
   TimerInterface callback;

   /**
    * default constructor
    */
   public Timer() {
     interval = 1000;
   }

   /**
    * constructor
    * @param long millisecond number of millisecond for the interval
    * @param TimerInterface callback reference to the interface
    */
   public Timer(long milliseconds, TimerInterface callback) {
     interval = milliseconds;
     this.callback = callback;
   }

   /**
    * return milliseconds for an interval
    */
   public long getInterval() {
     return interval;
   }

   /**
    * set interval
    * @param long milliseconds number of milliseconds
    */
   public void setInterval(long milliseconds) {
     interval = milliseconds;
   }

   /**
    * starts the timer
    */
   public void start() {
     myThread = new Thread(this);
     myThread.start();
   }

   /**
    * stops the timer
    */
   public void stop() {
     myThread.stop();
     myThread = null;
   }

   /**
    * run
    */
   public void run() {
     while(true) {
       long sTime = System.currentTimeMillis();
       if (callback != null)
         callback.tick();
       long eTime = System.currentTimeMillis();
       long sleep = interval - (eTime-sTime);
       if (sleep <= 0)
         continue;
       try 
       {
           Thread.sleep(sleep);
       }
       catch (Exception e) 
       {
           System.out.println("Exception during sleep operation\n");
       }
     }
   }
}
